require 'rails_helper'

module Fettle
  RSpec.describe HealthController, type: :controller do
    routes { Fettle::Engine.routes }

    describe '#show' do
      context 'when all checks are passing' do
        before do
          allow(Checker).to receive(:run).and_return(CheckResult.new(true))
        end

        it 'returns a 200 OK' do
          get :show

          expect(response).to have_http_status(:ok)
        end

        context 'and it is a liveness check' do
          before do
            allow(Checker).to receive(:run).and_return(LivenessCheckResult.new(true))
          end

          it 'returns a 200 OK' do
            get :show, params: { tag: :liveness }

            expect(response).to have_http_status(:ok)
          end
        end
      end

      context 'when some checks are failing' do
        before do
          allow(Checker).to receive(:run).and_return(CheckResult.new(false))
        end

        it 'returns a 503 Service Unavailable' do
          get :show

          expect(response).to have_http_status(:service_unavailable)
        end

        context 'when the tag passed is readiness' do
          before do
            allow(Checker).to receive(:run).and_return(LivenessCheckResult.new(false))
          end

          it 'returns a 200 OK' do
            get :show, params: { tag: :liveness }

            expect(response).to have_http_status(:ok)
          end
        end
      end
    end
  end
end
