require 'spec_helper'
require 'fettle/checker'

module Fettle
  RSpec.describe Checker do
    describe '.run' do
      subject(:result) { described_class.run(tag: tag) }

      before do
        allow(Configuration).to receive(:checks).and_return(checks)
      end
      let(:checks) do
        [ check ]
      end

      let(:tag) { nil }

      let(:check) do
        Class.new do
          def self.call
            true
          end
        end
      end

      context 'when all checks are successful' do
        it 'returns a successful CheckResult' do
          expect(result).to be_an_instance_of(CheckResult)
          expect(result.healthy?).to be_truthy
        end

        context 'and the liveness tag is passed' do
          let(:tag) { 'liveness' }

          it 'returns a successful LivenessCheckResult' do
            expect(result).to be_an_instance_of(LivenessCheckResult)
            expect(result.healthy?).to be_truthy
          end
        end
      end

      context 'when a check fails' do
        let(:check) do
          Class.new do
            def self.call
              false
            end
          end
        end

        it 'returns a failed CheckResult' do
          expect(result).to be_an_instance_of(CheckResult)
          expect(result.healthy?).to be_falsey
        end

        context 'and the liveness tag is passed' do
          let(:tag) { 'liveness' }

          it 'returns a failed LivenessCheckResult' do
            expect(result).to be_an_instance_of(LivenessCheckResult)
            expect(result.healthy?).to be_falsey
          end
        end
      end

      context 'when a check errors' do
        let(:check) do
          Class.new do
            def self.call
              raise
            end
          end
        end

        it 'returns a failed CheckResult' do
          expect(result).to be_an_instance_of(CheckResult)
          expect(result.healthy?).to be_falsey
        end

        context 'and the liveness tag is passed' do
          let(:tag) { 'liveness' }

          it 'returns a failed LivenessCheckResult' do
            expect(result).to be_an_instance_of(LivenessCheckResult)
            expect(result.healthy?).to be_falsey
          end
        end
      end
    end
  end
end
