require 'rails_helper'
require 'fettle/checks/active_record_check'

module Fettle
  module Checks
    RSpec.describe ActiveRecordCheck do
      describe '.call' do
        subject(:check) { described_class.call }

        context 'when a connection can be established' do
          before do
            allow(ActiveRecord::Base)
              .to receive(:connected?).and_return(true)
          end

          it { is_expected.to be_truthy }
        end

        context 'when a connection can not be established' do
          before do
            allow(ActiveRecord::Base)
              .to receive(:connected?).and_return(false)
          end

          it 'raises a failed check error' do
            expect { check }.to raise_error(
              FailedCheck,
              'Failed to establish database connection'
            )
          end
        end
      end
    end
  end
end
